@@ -5,6 +5,7 @@ use warnings;
 
 use HTTP::Date;
 use APR::Table           ();
+use URI::Escape          ();
 use Apache2::RequestUtil ();
 use Apache2::Log         ();
 use Apache2::RequestRec  ();
@@ -44,7 +45,7 @@ use constant {
 };
 
 BEGIN {
-    our $VERSION = join( '.', 1, map{ sprintf( '%03d', $_ - 18 ) } ( '$Revision: 118 $' =~ /(\d+)/g ) );
+    our $VERSION = join( '.', 1, map{ sprintf( '%03d', $_ - 18 ) } ( '$Revision: 121 $' =~ /(\d+)/g ) );
 };
 
 my ( $i, $x )       = ( 0, 0 );
@@ -59,6 +60,8 @@ my $APPEND_INC_NAME = 0;
 my $SEPARATOR       = '~';
 my $DOC_ROOT        = '';
 my %CONTENT_TYPES   = ( 'js'=> 'text/javascript', 'css' => 'text/css', );
+my %VARS;
+
 
 {
     my %cache;
@@ -99,6 +102,8 @@ my %CONTENT_TYPES   = ( 'js'=> 'text/javascript', 'css' => 'text/css', );
         ( $location, $file, $type ) = $uri =~ /^(.*)\/(.*)\.(js|css)$/;
         my $root                    = $DOC_ROOT || $r->document_root();
 
+        _substitute_vars( \$location, \$file ) if ( %VARS );
+
         foreach my $input ( split( $SEPARATOR, $file ) ) {
             $input     =~ s/\./\//g;
             $content  .= ( _load_content( $root, $location, $input, $type ) || '' );
@@ -145,19 +150,22 @@ my %CONTENT_TYPES   = ( 'js'=> 'text/javascript', 'css' => 'text/css', );
 
     {
         my %loaded;
-        my %vars;
 
         sub _init {
             my( $r ) = @_;
 
             %loaded  = ();
-            %vars    = map { my( $k, $v ) = split(/=/); $k => ( $v || '' ) } split( /[&;]/, $r->args() );
+            %VARS    = map {
+                my( $k, $v ) = split(/=/);
+                $k => URI::Escape::uri_unescape( ( $v || '' ) )
+            } split( /[&;]/, $r->args() );
         }
 
         sub _load_content {
             my ( $root, $location, $file_name, $type ) = @_;
+            return unless ( $file_name );
 
-            _substitute_vars( \$location, \$file_name ) if ( %vars );
+            _substitute_vars( \$location, \$file_name ) if ( %VARS );
 
             $LOG->debug( "\$location = $location" );
             $location =~ s/\/$//g;
@@ -211,8 +219,8 @@ my %CONTENT_TYPES   = ( 'js'=> 'text/javascript', 'css' => 'text/css', );
             foreach my $string ( @_ ) {
                 while( $$string =~ /\{\s*([\w-]+)\s*\}/g ) {
                     my $varname = $1;
-                    next unless defined $vars{$varname};
-                    $$string =~ s/\{\s*$varname\s*\}/$vars{$varname}/g;
+                    next unless defined $VARS{$varname};
+                    $$string =~ s/\{\s*$varname\s*\}/$VARS{$varname}/g;
                 }
             }
         }
@@ -563,6 +571,44 @@ The generall usefulness of the advanced URI protocol is to combine
 files that are seemingly not dependant upon one another.  See
 the EXAMPLES section for more details on this.
 
+=head1 TROUBLESHOOTING AND COMMON ISSUES
+
+=item1 File name too large errors
+
+There is a common barrier with Apache (v1+, v2+) servers (also others, 
+though Apache is the most susceptible) where you will be served a 
+Forbidden page rather than the requested JS/CSS source when a file
+name grows larger than 256 characters.  When combining multiple files
+with deep relative path locations, this is an easy threshold to cross.
+A work-around to this issue is by using variable substitution to
+put the file-name as a query string parameter rather than the file name,
+where the total lenght of a URI isn't limited as tightly as the file name
+itself.
+
+For example:
+
+    http://yourdomain.com/js/dir.dir2.dir<.. 256 characters later>.dirn.file.js
+
+Can be altered to:
+
+    http://yourdomain.com/{file}/?file=js/dir.dir2.dir<.. 256 characters later>.dirn.file.js
+
+
+The big, big caveat being that the current implementation of this module
+will only process varaibles once and in a single-dimension.  Therefore, if you
+are troubleshooting a URL that already utilizes variable substitution, such as:
+
+    http://yourdomain.com/js/{lang}.dir.dir2.dir<.. 256 characters later>.dirn.file.js?lang=en
+
+And you attempt to convert it to:
+
+    http://yourdomain.com/{file}/?file=js/{lang}.dir.dir2.dir<.. 256 characters later>.dirn.file.js&lang=en
+
+The "lang" variable will not be substitued as it lies within the querystring rather than the path.  For
+these cases, we recommend you put the secondary variable substitution into an "Include" rather than 
+the path, which should be sufficient in most cases.
+
+
 =head1 KNOWN ISSUES
 
 =over
